<?php

declare(strict_types=1);

namespace Drupal\entity_id_manager\Commands;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\entity_id_manager\EntityIdManager;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\Console\Input\InputInterface;

/**
 * @property InputInterface $input
 * @property EntityIdManager $entityIdManager
 * @property EntityTypeManagerInterface $entityTypeManager
 */
trait EntityIdAskTrait
{
  protected function askId(string $entityType): ?int
  {
    $storage = $this->entityTypeManager->getStorage($entityType);
    $column = $this->entityTypeManager->getDefinition($entityType)->getKey('id');
    $query = $storage->getQuery()
      ->accessCheck(FALSE)
      ->sort($column);
    if($entityType === 'user') {
      $query->condition('uid', 1, '>');
    }
    $ids = $query->execute();

    if (empty($ids)) {
      $this->getlogger('entity_id_manager')->warning('Entity tables empty -> Nothing to change of the id of.');
      return null;
    }

    foreach ($ids as $id) {
      $entity = $storage->load($id);
      assert(assertion: $entity instanceof ContentEntityInterface);
      $bundle = $entity->bundle();
      $label = match ($entity->getEntityTypeId()) {
        'commerce_product_variation' => $entity->getSku(),
        default => $entity->label()
      };
      $choices["id:" . $id] = t("(id: $id, bundle: $bundle) @label", ['@label' => $label]);
    }

    $result = $this->io()->choice('Old Entity id', $choices);
    return (int) explode(':', $result)[1];
  }

  protected function askIdUnused(string $entityType): int
  {
    $ids_unused = $this->entityIdManager->getAvailableIds($entityType);

    foreach ($ids_unused as $id) {
      $choices["id:" . $id] = "(id: $id)";
    }

    $result = $this->io()->choice('New Entity id', $choices);
    return (int) explode(':', $result)[1];
  }
}