<?php

declare(strict_types=1);

namespace Drupal\entity_id_manager\Commands;

use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\entity_id_manager\EntityIdManager;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drush\Commands\field\EntityTypeBundleAskTrait;

class EntityIdCommands extends DrushCommands
{
  use EntityTypeBundleAskTrait;
  use EntityIdAskTrait;

  use LoggerChannelTrait;

  const CHANGE_ID = 'entity:change-id';
  const CHANGE_ID_ALIAS = 'ecid';

  const MERGE_ID = 'entity:merge-id';
  const MERGE_ID_ALIAS = 'emid';

  const NUM_ID_NEW_OFFER = 25;


  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly EntityIdManager $entityIdManager,
    protected CacheBackendInterface $cacheEntity
  ) {
  }

  /**
   * Changes the id of an entity
   */
  #[CLI\Command(name: self::CHANGE_ID, aliases: [self::CHANGE_ID_ALIAS])]
  #[CLI\Argument(name: 'entityType', description: 'Entity type')]
  #[CLI\Option(name: 'show-machine-names', description: 'Show machine names instead of labels in option lists.')]
  // #[CLI\Argument(name: 'id', description: 'Entity id')]
  // #[CLI\Argument(name: 'id_new', description: 'New Entity id')]
  #[CLI\Usage(name: self::CHANGE_ID . ' node 20 11', description: 'Usage description')]
  public function changeContentEntityId(?string $entityType = null, ?int $idOld = null, ?int $idNew = null, array $options = [
      'format' => 'table',
  ]): void
  {
    
    // # Prompt user
    // if not all arguments have been passed via drush
    $this->input->setArgument('entityType', $entityType = $entityType ?? $this->askEntityType());
    $this->input->setArgument('idOld', $idOld = $idOld ?? $this->askId($entityType));
    $this->input->setArgument('idNew', $idNew = $idNew ?? $this->askIdUnused($entityType));
    
    $storage = $this->entityTypeManager->getStorage($entityType);
    $label = $storage->load($idOld)->label();

    // # Execute
    $this->entityIdManager->changeEntityId($entityType, $idOld, $idNew);

    // # Report  
    $this->getLogger('entity_id_manager')->warning("\"$label\" id changed: $idOld -> $idNew");

    // # Clear cache
    // @optimize Clear the cache with more precision. Something like $this->cacheEntity->delete('entity')
    $this->cacheEntity->invalidateMultiple(['entity']);
  }

  /**
   * Example usage for merge content: Merge the entities owned by one user (2nd uid argument) to another user (1st uid argumennt).)
   */
  /* #[CLI\Command(name: self::MERGE_ID, aliases: [self::MERGE_ID_ALIAS])]
  #[CLI\Argument(name: 'entity_type_id', description: 'The entity type to merge')]
  #[CLI\Argument(name: 'id', description: 'The id to merge into')]
  #[CLI\Argument(name: 'id_discarded', description: 'The id to discared')]
  #[CLI\Usage(name: self::MERGE_ID . ' node 20 11', description: 'Usage description')]
  public function mergeContentEntities($entity_type_id, $id, $id_discarded): void
  {
    $storage = $this->entityTypeManager->getStorage($entity_type_id);
    $discarded = $storage->load($id_discarded);
    $label_primary = $storage->load($id)->label();
    $label_discarded = $discarded->label();

    $this->entityIdManager->updateEntityReferenceIds($entity_type_id, $id_discarded, $id);
    $discarded->delete();

    $this->getlogger('entity_id_manager')->warning("Merged \"$label_discarded\" into \"$label_primary\"");

    $this->cacheEntity->invalidateMultiple(['entity']);
  } */
}
